#------------------------------------------------------------------------------
#    plSokoban Suite - perl OOP sokoban implementation
#    Copyright (c) 2002 Arne Bochem <webmaster@arneb.de>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#------------------------------------------------------------------------------

use strict;
use Sokoban::Element;

package Sokoban::Element::Boulder;
@Sokoban::Element::Boulder::ISA = qw( Sokoban::Element );

sub boulder
{
	return(1);
}

sub move
{
	my ($self, $direction) = @_;
	my %dirmap = (
	              'up' => [0, -1],
	              'down' => [0, 1],
	              'left' => [-1, 0],
	              'right' => [1, 0]
	             );

	my ($x, $y) = ($self->{'_x'} + $dirmap{$direction}->[0], $self->{'_y'} + $dirmap{$direction}->[1]);
	my $element = $self->{'_parent'}->get_element($x, $y);

	$element or return(0);
	$element->free() or return(0);
	$element->redefine('x');
	$self->redefine('@');
}

sub redefine
{
	my ($self, $element_type) = @_;

	($self->{'_target'} and ($element_type ne 'x')) and $self->{'_parent'}->boulder_remove();
	$self->{'_parent'}->redefine($self, $element_type);
}

sub element_type
{
	my $self = shift;
	$self->{'_target'} and return('X');
	return $self->{'_element_type'};
}

1;
